      program  bayes
c perform Bayesian analysis of imaginary time correlation functions
c D. Ceperley, June 1994
      implicit real*8 (a-h,o-z)
       parameter (mne=30,mstates=1,mtimes=20,mderv=1
     +,msq=(mstates*(mstates+1))/2,ms1=mstates+1
     +,mvl=mtimes*mderv,mdata=mvl*msq,nprior=1,nh=400,mnh=6)

c ne/mne = total number of energy values
c
c nproj = 0 for e(-tau H) and 1 for (1+tau H)^-1
c tau etrial are defined above
c nskip and ntimes describe the times that h is written out
c nderv is the number of derivatives written out
c nstates/mstates is the total number of excited states
c hmc is the MC computed correlation functions
c nsig are the number of significant elements after the svd on covar
c u is the significant directions of data
c
c e are the fitted energy levels, ne are the number of them
c ndata/mdata = nderv*nstates**2*ntimes
      dimension delnewe(mdata),dele(mne,mdata),delta(mdata)
     +,step(ms1,mne),ynew(msq,mdata),y(msq,mne,mdata)
     +,nacc(ms1,mne),ntac(ms1,mne),av(ms1,mne),avsq(ms1,mne)
     +,dnew(ms1),s(mvl,mne),snew(mvl)
     +,hmc(mdata),u(mdata,mdata),pot0(mne)
     +,d(ms1,mne),xx(mdata),yy(msq,mne,mdata),dna(mne)
     +,iseed(2),histo(nh,mnh),elow(mnh),dtbl(mnh)
     +,dhisto(nh,mnh),emax(mnh),emin(mnh)
       iseed(1)=44
       iseed(2)=66
c reasonable spreads in alpha
       alpha0=.0000001
       alpha1=10000.
       call iranset(iseed)

c read mc data
       call readdata(nderv,ntimes,nstates,tau,etrial,nskip,hmc
     + ,mdata,ndata,nproj)

      read (3,*) nsig,n
      write (*,*) ' number of data points ',nsig,n
      do 750 i=1,nsig  
750   read (3,751) (u(k,i),k=1,n) 
      do 10 il=1,nsig
      xx(il)=0.
      do 10 j=1,n
10    xx(il)=xx(il)+hmc(j)*u(j,il)

      read (3,*) ne
      write (*,*) ' number of energies ',ne
      if(ne.ge.mne)stop
      do 900 i=1,ne
      read (3,19) d(nstates+1,i),(d(is,i),is=1,nstates)
19    format(5e12.5)
      write (6,19) (d(is,i),is=1,nstates+1)
900   continue


      nhisto=nstates+1
      nfsm=nstates+1
      ns1=nstates+1
      ns2=(nstates*(nstates+1))/2
      ntd=nderv*ntimes

      write (*,*)' input:nsteps nbbe  nblocks'
      read (*,*) nsteps,nbbe,nblocks
      nvary=1
      write (*,*)' input old parameters? yes =1'
      read (*,*) ifold
      if(ifold.eq.1) then
        do 965 ie=1,ne
        read(4,*) (d(is,ie),is=1,ns1)
965     read(4,*) (step(is,ie),is=1,ns1)
        read (4,*) alpha
        read (4,*) nhisto,(emax(i),emin(i),i=1,nhisto)
        do 966 i=1,nhisto
        dtbl(i)=(nh-1)/(3.*(emax(i)-emin(i)))
966     elow(i)=2*emin(i)-emax(i)

      else
         do 955 ie=1,ne
         do 955 is=1,ns1
955      step(is,ie)=.005 
         alpha=10.
         do 956 i=1,nhisto
         dtbl(i)=1./nh
         elow(i)=d(ns1,i)-.5
         emax(i)=d(ns1,i)
956      emin(i)=d(ns1,i)
         emax(nhisto)=d(ns1,ne+1)
      endif


      avchi=0.
      avalpha=0.
c  initial step size
      do 920 ie=1,ne
      dna(ie)=0.
      do 920 is=1,ns1
      ntac(is,ie)=0
      av(is,ie)=0.
920   avsq(is,ie)=0.
      chisq=0.

c loop over the number of blocks
      do 300 iblk=-nbbe+1,nblocks

      chibl=0.
      do 301 ie=1,ne
      do 301 is=1,ns1
301   nacc(is,ie)=0

c loop over number of steps/block
      do 112 isteps=1,nsteps

      chiold=chisq
      chisq=0.
      do 4 iep=1,ne
4     call pfunc(nproj,tau,d(nstates+1,iep),nskip,nderv,ntimes,s(1,iep)
     +  ,beta)
      do 2 il=1,nsig
      delta(il)=-xx(il)
      do 1 iep=1,ne
      call findy(s(1,iep),y(1,iep,il),il,ntd,nstates,u,mdata)
c     dele(iep,il)=trace(y(1,iep,il),d(1,iep),nstates)
      dele(iep,il)=y(1,iep,il)*d(1,iep)
1     delta(il)=delta(il)+dele(iep,il)
2     chisq=chisq+delta(il)**2
c check chi**2
      if(abs(chisq-chiold).gt..001) then
       write (*,*) isteps,' chi**2 ',chisq,chiold
      endif

      priortot=0.
      do 110 ie=1,ne

      dnorm=0
      do 101 is=1,nstates
101   dnorm=dnorm+d(is,ie)**2
      if(iblk.gt.0)dna(ie)=dna(ie)+dnorm

c put energies in histogram
      ia=min(nhisto,ie)
      emax(ia)=max(emax(ia),d(ns1,ie))
      emin(ia)=min(emin(ia),d(ns1,ie))
      l=max(0.,min(nh-1.,dtbl(ia)*(d(ns1,ie)-elow(ia)) ))+1.
      dhisto(l,ia)=dhisto(l,ia)+dnorm
      histo(l,ia)=histo(l,ia)+1.


c subtract out Y component
      do 104 il=1,nsig
104   delta(il)=delta(il)-dele(ie,il)

      nsp=ns1
      if(ie.gt.nvary)nsp=nstates
      do 100 is=1,nsp
      enew=d(is,ie)+step(is,ie)*(ranf()-.5)

      if(is.eq.ns1) then
c this is really an energy move
      dprior=0.

      call pfunc(nproj,tau,enew,nskip,nderv,ntimes,snew,beta)
      do  il=1,nsig
      call findy(snew,ynew(1,il),il,ntd,nstates,u,mdata)
c      delnewe(il)=trace(ynew(1,il),d(1,ie),nstates)
       delnewe(il)=ynew(1,il)*d(1,ie)
      endo

      else
c this is a wavefunction move
      if(enew.gt.0) then
        do 54 ii=1,nstates
54      dnew(ii)=d(ii,ie)
        dnew(is)=enew
        do  il=1,nsig
c      delnewe(il)=trace(y(1,ie,il),dnew,nstates)
         delnewe(il)=y(1,ie,il)*dnew
        enddo
        dprior=alpha*(enew-d(ii,ie)-enew*log(enew)+d(ii,ie)*log(d(ii,ie))

      else
         dprior=-100
         chinew=chisq
      endif
      endif

      chinew=0.
      do 50 il=1,nsig
50    chinew=chinew+ (delta(il)+delnewe(il))**2

      pacc=min(exp(dprior-.5*(chinew-chisq)),1.)
      if(iblk.gt.0)then
      prej=1.-pacc
      av(is,ie)=av(is,ie)+    pacc*enew    +prej*d(is,ie)
      avsq(is,ie)=avsq(is,ie)+pacc*enew**2 +prej*d(is,ie)**2
      chibl=chibl            +pacc*chinew + prej*chisq
      endif

c accept or rject
      if(pacc.gt.ranf()) then
      chisq=chinew
      dnorm=dnorm+deldnorm

        if(is.eq.ns1)then
        do 65 ii=1,ns2
        do 65 il=1,nsig
65      yy(ii,ie,il)=ynew(ii,il)
        priorold=priornew
        endif

      do 67 il=1,nsig
67    dele(ie,il)=delnewe(il)
      nacc(is,ie)=nacc(is,ie)+1

      d(is,ie)=enew
      endif
100   continue
c restore delta
      do 105 il=1,nsig
105   delta(il)=delta(il)+dele(ie,il)
      priortot=priortot+priorold
110   continue


c now readjust alpha using priortot
      alpha=samx(priortot/alpha,alpha0,alpha1)
      if(iblk.gt.0) avalpha=avalpha+alpha
112   continue

c print out acceptance ratios, energies and error bars.
      n=iblk*nsteps
      do 200 ie=1,ne
      if(iblk.gt.0)dnorm=dna(ie)/n
      nsp=ns1
      if(ie.gt.nvary)nsp=nstates
      do 200 is=1,nsp
c adjust the step size
      ar=float(nacc(is,ie))/nsteps
      ntac(is,ie)=ntac(is,ie)+nacc(is,ie)
      if(ar.gt..8) step(is,ie)=1.5*step(is,ie)
      if(ar.lt..2) step(is,ie)=.7*step(is,ie)
      if(iblk.gt.0) then
      dbar=av(is,ie)/n
      err=sqrt(abs(avsq(is,ie)/n-dbar**2))
      if(is.eq.ns1)then 
       dbar=dbar+etrial
      write (*,201) is,ie,ntac(is,ie),step(is,ie),ar,dbar,err,dnorm
201   format(2i2,i7,e10.3,f6.3,3e14.6)
      endif
      endif
200   continue
      if(iblk.gt.0) then
      chibl=chibl/(nsteps*ne*ns1)
      avchi=avchi+chibl
      write (*,*)' average chisq ',avchi/iblk
      write (*,*)' averages alpha ',avalpha/(iblk*nsteps)
      endif
300   continue
        rewind 4
        do 960 ie=1,ne
        write(4,44) (d(is,ie),is=1,ns1)
960     write(4,44) (step(is,ie),is=1,ns1)
        write (4,44) alpha
        write (4,*) nhisto,(emax(i),emin(i),i=1,nhisto)
44      format(5e12.5)
c put out condensed table of histrograms
      do 1020 ie=1,nhisto
      do 1021 l=1,nh
1021  if(histo(l,ie).gt.0) go to 1022
1022  nfirst=l
      do 1023 l=nh,1,-1
1023  if(histo(l,ie).gt.0) go to 1024
1024  nlast=l
      e0=elow(ie)+(nfirst-.5)/dtbl(ie)+etrial
      e1=elow(ie)+(nlast-.5)/dtbl(ie)+etrial

      if(ie.le.nstates) then
      write (2,*) 'RANK 1 ',nlast-nfirst+1
      write (2,1026)e0,e1
1026  format('GRID 1 LINEAR ',2f12.5)
      write (2,*)' BEGIN DATA '
      write (2,1025) (histo(l,ie),l=nfirst,nlast)
1025  format(10f10.0)
      else
      write (2,*) 'RANK 1 ',nlast-nfirst+1
      write (2,1026)max(e0-etrial,.01),e1-etrial
      write (2,*)' BEGIN DATA '
      write (2,1027) (dhisto(l,ie),l=nfirst,nlast)
1027  format(5e11.4)
      endif

1020  continue
      stop
      end
   
      subroutine findy(s,y,il,ntd,nstates,u,mdata)
      implicit real*8 (a-h,o-z)
      dimension y(2),s(ntd),u(mdata,mdata)
      j=0
      iab=0
      do 10 ia=1,nstates
      do 10 ib=1,ia
      iab=iab+1
      y(iab)=0.
      do 10 k=1,ntd
      j=j+1
      y(iab)=y(iab)+u(j,il)*s(k)
10    continue
      return
      end

      subroutine readdata(nderv,ntimes,nstates,tau,etrial,nskip,hmc
     + , mdata,ndata,nproj)
      implicit real*8 (a-h,o-z)
      character name*14,namet*14,title*60
      dimension hmc(mdata)
      logical ifex
41    write (*,*) ' input run id?'
      read (*,287) name
287   format(a14)
        ln=index(name,' ')-1
      namet=name(1:ln)//'.me'
      lnt=ln+3
c does this file exist?
        inquire(file=namet(1:lnt),exist=ifex)
        if(ifex) go to 40
         write (*,*)' file does not exist '
         go to 41
40     open(3,file=name(1:ln)//'.me')
      open(4,file=name(1:ln)//'.by')
      open(2,file=name(1:ln)//'.hs')
      write (*,*) ' file opened'

      read (3,*) nderv,nstates,ntimes,nskip,nproj
      write (*,91)nderv,nstates,ntimes,nskip,nproj
c     if(nstates.gt.mstates)stop
c     if(ntimes.gt.mtimes)stop
c     if(ndervt.gt.mderv)stop
      if(nproj.gt.3.or.nproj.lt.1)stop

      read (3,*) tau,etrial
      write (*,*) tau,etrial
      ndata=(nderv*ntimes*nstates*(nstates+1))/2
      write (*,*)' total number of data elements ',ndata
      if(ndata.gt.mdata) then
        write (*,*)' not enough space for data ', ndata,mdata
        stop
      endif
      read (3,*) (hmc(i),i=1,ndata)
91    format(6i10)

      ntd=ntimes*nderv
      ndata=(ntd*nstates*(nstates+1))/2

      return
      end

      subroutine pfunc(nproj,tau,e,nskip,nderv,ntimes,s,beta)
      implicit real*8 (a-h,o-z)
      dimension s(nderv,ntimes)

      x=tau*e
      fp=0
      fi=1.
      if(nproj.eq.1) then
         xi=1./(1.+x)
         f=xi**nskip
      elseif(nproj.eq.2) then
         f=exp(-nskip*x)
         fp=exp(-beta*e)
         fi=1./f
      elseif(nproj.eq.3) then
         f=(1-x)**nskip
      endif

c fk=f**k
      fk=1.
      fpk=fp
      do 10 k=0,ntimes-1
c fkl=f**k * e**l
      fkl=fk+fpk
      do 20 l=0,nderv-1
      s(l+1,k+1)=fkl
      fkl=fkl*e
20    continue
      fk=fk*f
      fpk=fpk*fi
10    continue

      return
      end

      function samx(s,x0,x1)
      implicit real*8 (a-h,o-z)
c returns a random number distributed according to e(-x)/x in (x0,x1)
      call eint(s*x0,e0,de0)
      call eint(s*x1,e1,de1)
      c=ranf()*(e0-e1)+e1
      xdn=x0
      xup=x1
      samx=.5*(x0+x1)
1     call eint(s*samx,e,de)
      dx=-(e-c)/(s*de)
      if(e.gt.c) then
          xdn=samx
          dx=min(dx,.8*(xup-xdn))
      else
        xup=samx
        dx=max(dx,-.8*(xup-xdn))
      endif
       samx=samx+dx
      if(abs(dx).gt.1.e-5*samx) go to 1
      return
      end

      subroutine eint(x,e1,de1)
      implicit real*8 (a-h,o-z)
      de1=-exp(-x)/x
      if(x.lt.1) then
         e1=-log(x)-.57721566+x*(.99999193+x*(-.24991055
     + +x*(.05519968+x*(-.00976004+x*.00107857) ) ) )
      else
        e1=-de1*(.250621+x*(2.334733+x))/(1.681534+x*(3.330657+x))
      endif
      return
      end
